//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

/**
 * @brief Allow socket to upload limited number of bytes per second
 *
 * This function is called by MOD_GetDownloadLimit()
 * @return void
 */
void CEMSocket::MOD_RefreshDownloadLimit()
{	
	CMilliSeconds	timeDiff = CMilliSeconds(200);
	if (downloadLimitLastCall.IsTickSet())
		timeDiff = downloadLimitLastCall.Elapsed();
	if (timeDiff >= CMilliSeconds(20) && downloadLimitEnable) // Don't update to often
	{
		downloadLimitLastCall.SetCurrentTick();
		// MOD BEGIN netfinity: Accumulate download limits
		downloadLimit += downloadRate * timeDiff;	
		if(downloadLimit > downloadRate * SEC2MS(2) && downloadLimit > 4500) // Allow a maximum of 2.0 sec to accumulate or 3 * MTU
			downloadLimit = max(downloadRate * SEC2MS(2), 4500);
		// MOD BEGIN netfinity
	}
}

/**
 * @brief Get the number of bytes allowed to download before reaching the limit
 *
 * This function is called by OnReceive()
 * @return CBytes - Bytes allowed to download before limit is reached
 */
CBytes CEMSocket::MOD_GetDownloadLimit()
{
	if (!downloadLimitEnable)
	{
		return _I32_MAX; // netfinity: Just temporary until CBytes implementes infinite numbers
	}
	else
	{
		MOD_RefreshDownloadLimit();
		return max(downloadLimit, CBytes(0));
	}
}

/**
 * @brief Set the number of bytes used of the download limit
 *
 * This function is called by OnReceive()
 * @param bytes - Number of bytes used of the limit
 */
void CEMSocket::MOD_SetUsedDownloadLimit(const CBytes& bytes)
{
	// Only bother if limit is activated
	if (downloadLimitEnable)
	{
		downloadLimit -= bytes;
	}
}

/**
 * @brief Allow socket to download limit number of bytes per second
 *
 * This function is called approx. every 0.1 second by CPartFile::ProcessDL()
 * @param datarate Number of bytes to send during this 0.1 second slot given in bytes per second 
 * @return void
 */
void CEMSocket::MOD_SetDownloadLimitByRate(const CBytesPerSecond& datarate)
{	
	/*receiveLocker.Lock();
	CMilliSeconds	timeDiff = CMilliSeconds(200);
	if (downloadLimitLastCall.IsTickSet())
		timeDiff = downloadLimitLastCall.Elapsed();
	downloadLimitEnable = true;	
	if (timeDiff >= CMilliSeconds(20))
	{
		downloadLimitLastCall.SetCurrentTick();
		downloadRate = datarate;
		if (!(timeDiff >= SEC2MS(0) && timeDiff <= SEC2MS(1)))
			timeDiff = SEC2MS(1);
		// MOD BEGIN netfinity: Accumulate download limits
		downloadLimit += datarate * timeDiff;
		if(downloadLimit > downloadRate * SEC2MS(2) && downloadLimit > 4500) // Allow a maximum of 2.0 sec to accumulate or 3 * MTU
			downloadLimit = max(downloadRate * SEC2MS(2), 4500);
		// MOD BEGIN netfinity
	}
	receiveLocker.Unlock();*/
	
	MOD_SetDownloadRate(datarate);
	// CPU load improvement
	if(MOD_GetDownloadLimit() > 0 && pendingOnReceive == true){
		OnReceive(0);
	}
}

/**
 * @brief Allow socket to download limit number of bytes per second
 *
 * @param datarate - Number of bytes per second to receive on the socket each second
 * @return void
 */
void CEMSocket::MOD_SetDownloadRate(const CBytesPerSecond& datarate)
{
	// Be careful with here, these datamembers are used by different threads
	downloadRate = datarate;
	if (!downloadLimitEnable)
	{
		downloadLimit = 0;
	}
	downloadLimitEnable = true;
}

/**
 * @brief Allow socket to download as much data as possible
 *
 * @return void
 */
void CEMSocket::DisableDownloadLimit(){
	receiveLocker.Lock();
	downloadLimitEnable = false;
	receiveLocker.Unlock();

	// CPU load improvement
	if(pendingOnReceive == true){
		OnReceive(0);
	}
}

/**
 * @brief Allow socket to upload limited number of bytes per second
 *
 * This function is called by MOD_GetUploadLimit()
 * @return void
 */
void CEMSocket::MOD_RefreshUploadLimit()
{	
	CMilliSeconds	timeDiff = CMilliSeconds(200);
	if (downloadLimitLastCall.IsTickSet())
		timeDiff = uploadLimitLastCall.Elapsed();
	if (timeDiff >= CMilliSeconds(20) && uploadLimitEnable) // Don't update to often
	{
		uploadLimitLastCall.SetCurrentTick();
		// MOD BEGIN netfinity: Accumulate upload limits
		uploadLimit += uploadRate * timeDiff;	
		if(uploadLimit > uploadRate * SEC2MS(2) && uploadLimit > 4500) // Allow a maximum of 2.0 sec to accumulate or 3 * MTU
			uploadLimit = max(uploadRate * SEC2MS(2), 4500);
		// MOD BEGIN netfinity
	}
}

/**
 * @brief Get the number of byte allowed to upload before reaching the limit
 *
 * This function is called by Send()
 * @return CBytes - Bytes allowed to upload before limit is reached
 */
CBytes CEMSocket::MOD_GetUploadLimit()
{
	if (!uploadLimitEnable)
	{
		return _I32_MAX; // netfinity: Just temporary until CBytes implementes infinite numbers
	}
	else
	{
		MOD_RefreshUploadLimit();
		return max(uploadLimit, CBytes(0));
	}
}

/**
 * @brief Set the number of bytes used of the upload limit
 *
 * This function is called by Send()
 * @param bytes Number of bytes used of the limit
 */
void CEMSocket::MOD_SetUsedUploadLimit(const CBytes& bytes)
{
	// Only bother if limit is activated
	if (uploadLimitEnable)
	{
		uploadLimit -= bytes;
	}
}

/**
 * @brief Allow socket to upload limit number of bytes per second
 *
 * This function is called approx. every 0.1 second by CUploadQueue::Process()
 * @param datarate Number of bytes per second to send by upload throttler for this socket
 * @return void
 */
void CEMSocket::MOD_SetUploadRate(const CBytesPerSecond& datarate)
{
	// Be careful with here, these datamembers are used by different threads
	uploadRate = datarate;
	if (!uploadLimitEnable)
	{
		uploadLimit = 0;
	}
	uploadLimitEnable = true;
}

/**
 * @brief Allow socket to upload as much data as possible
 *
 * @return void
 */
void CEMSocket::MOD_DisableUploadLimit(){
	uploadLimitEnable = false;
}

/**
 * @brief Get and reset number of bytes received by socket since last call
 *
 * @return CBytes - Number of bytes received by socket since last call
 */
CBytes CEMSocket::MOD_GetReceivedBytesSinceLastCallAndReset()
{
	CBytes	result = m_actualBytesReceived;
	m_actualBytesReceived = 0;
	return result;
}
